from typing import Any, Generator

import imaplib

from ut_log.log import LogEq

TyAny = Any
TyBool = bool
TyIMAP4 = imaplib.IMAP4 | imaplib.IMAP4_SSL
TyDic = dict[Any, Any]
TyGenStr = Generator[str, None, None]
TyMsg = bytes | Any
TyStr = str
TnAny = None | Any


class MailRcv:
    """ Receive Email
    """
    @staticmethod
    def connect(d_rcv: TyDic) -> TyIMAP4:
        _sw_ssl: TyBool = d_rcv.get('sw_ssl', True)
        _host = d_rcv.get('host', 'imap.gmail.com')
        # Connect to the server
        if _sw_ssl:
            _mail: TyIMAP4 = imaplib.IMAP4_SSL(_host)
        else:
            _mail = imaplib.IMAP4(_host)
        return _mail

    @staticmethod
    def login(mail: TyIMAP4, d_rcv: TyDic) -> None:
        _from = d_rcv.get('from', 'bernd.stroehle@gmail.com')
        _password = d_rcv.get('password', '')
        mail.login(_from, _password)

    @staticmethod
    def yield_body(mail: TyIMAP4, message: TyStr) -> TyGenStr:
        import email
        _email_ids = message.split()
        for email_id in _email_ids:
            # Fetch the email by ID
            status, msg_data = mail.fetch(email_id, '(RFC822)')

            if not isinstance(msg_data, tuple):
                continue

            # Get the email content
            msg = msg_data[0][1]
            if not isinstance(msg, (bytes, bytearray)):
                continue

            # Parse the email
            email_message = email.message_from_bytes(msg)

            # Print the subject
            LogEq.debug("email_message['subject']", email_message['subject'])
            # Print the sender
            LogEq.debug("email_message['from']", email_message['from'])

            # Print the email body
            if email_message.is_multipart():
                for part in email_message.walk():
                    if part.get_content_type() == 'text/plain':
                        _payload_part: bytes | Any = part.get_payload(decode=True)
                        _body = _payload_part.decode()
                        yield _body
            else:
                _payload_msg: TyMsg = email_message.get_payload(decode=True)
                _body = _payload_msg.decode()
                yield _body



from typing import Any

import smtplib
import msal
from email import encoders

from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from ut_path.aododopath import AoDoDoPath
from ut_log.log import Log, LogEq

TyArr = list[Any]
TyDic = dict[Any, Any]
TyAoD = list[dict[Any, Any]]
TnAny = None | Any


class MailSnd:
    """
    Send Email
    """
    @staticmethod
    def sh_token_with_msal(d_send) -> Any:
        """
        Authenticat with Microsoft Authentication library
        """
        # Define your Azure AD app details
        client_id = d_send.get('client_id', '')
        client_secret = d_send.get('client_secret', '')
        tenant_id = d_send.get('tenant_id', '')
        url_authority = d_send.get('url_authority', 'https://login.microsoftonline.com')
        authority = f'{url_authority}/{tenant_id}'
        url_scope = [d_send.get('url_scope', 'https://graph.microsoft.com/.default')]
        scope = url_scope

        # Create a confidential client application
        app = msal.ConfidentialClientApplication(
            client_id,
            authority=authority,
            client_credential=client_secret
        )

        # Acquire a token
        result = app.acquire_token_for_client(scopes=scope)

        if 'access_token' in result:
            access_token = result['access_token']
            print("Token acquired successfully")
        else:
            error = result.get('error')
            error_description = result.get('error_description')
            a_msg = ['Failed to acquire token', error, error_description]
            msg = "\n".join(a_msg)
            raise Exception(msg)
        return access_token

    @staticmethod
    def add_attachements(msg, aopath, kwargs) -> None:
        for _path in aopath:
            LogEq.debug("_path", _path)

            # Attach the file
            _attachment = MIMEBase('application', 'octet-stream')
            with open(_path, 'rb') as fd:
                _attachment.set_payload(fd.read())
            encoders.encode_base64(_attachment)
            _attachment.add_header(
                    'Content-Disposition', f'attachment; filename={_path}')
            msg.attach(_attachment)

    @classmethod
    def send(cls, msg, d_send: TyDic) -> None:
        _from: str = d_send.get('from', '')
        _host: str = d_send.get('host', '')

        _authentication = d_send.get('authentication', 'base')
        match(_authentication):
            case 'oauth2', 'modern':
                _token = cls.sh_token_with_msal(d_send)
            case _:
                _token = d_send.get('password', '')

        _sw_ssl: bool = d_send.get('sw_ssl', True)
        if _sw_ssl:
            _port = d_send.get('ssl_port', 465)
            try:
                with smtplib.SMTP_SSL(_host, _port) as smtp:
                    smtp.login(_from, _token)
                    smtp.send_message(msg)
                    smtp.quit()
            except Exception:
                # print(f"An error occurred: {e}")
                raise
        else:
            _port = d_send.get('tls_port', 587)
            try:
                with smtplib.SMTP(_host, _port) as smtp:
                    smtp.starttls()
                    smtp.login(_from, _token)
                    smtp.send_message(msg)
                    smtp.quit()
            except Exception:
                # print(f"An error occurred: {e}")
                raise


class MailDoSnd:
    """
    Send Email
    """
    @staticmethod
    def create(d_snd: TyDic):
        _from = d_snd.get('from', '')
        _to = d_snd.get('to', '')
        _subject = d_snd.get('subject', '')
        _cc = d_snd.get('cc', '')
        _bcc = d_snd.get('bcc', '')
        # Create the email
        _msg = MIMEMultipart()
        _msg['From'] = _from
        _msg['To'] = _to
        _msg['Subject'] = _subject
        if _cc:
            _msg['Cc'] = _cc
        if _bcc:
            _msg['Bcc'] = _bcc
        return _msg

    @classmethod
    def send(cls, d_snd: TyDic, kwargs: TyDic) -> None:
        LogEq.debug("_d_snd", d_snd)
        _msg = cls.create(d_snd)

        _aodod_path: TyAoD = d_snd.get('paths', [])
        _aopath: TyArr = AoDoDoPath.sh_aopath(_aodod_path, kwargs)

        _a_body: TyArr = d_snd.get('body', [])
        LogEq.debug("_a_body", _a_body)
        _body: str = '\n'.join(_a_body)
        Log.debug(f"_body: {_body} before format")
        LogEq.debug("_aopath", _aopath)
        _body = _body.format(*_aopath)
        Log.debug(f"_body: {_body} after format")

        _msg.attach(MIMEText(_body, 'plain'))

        _sw_attachements: TyAoD = d_snd.get('sw_attachements', False)
        if _sw_attachements:
            MailSnd.add_attachements(_msg, _aopath, kwargs)

        # Send the email
        MailSnd.send(_msg, d_snd)



import sys

import ut_com.dec as dec
from ut_com.com import Com
from ut_cli.task import Task

from ap_mail.parms import Parms as MailParms
from ap_mail.task import Task as MailTask

from typing import Any

TyTup = tuple[Any, Any]


class App:

    t_parms_task: TyTup = (MailParms, MailTask)

    @classmethod
    @dec.handle_error
    @dec.timer
    def do(cls) -> None:
        Task.do(Com.sh_kwargs(cls, sys.argv))


if __name__ == "__main__":
    App.do()



from typing import Any

from ut_log.log import LogEq
from ut_path.path import Path
from ut_ioc.yaml_ import Yaml_

from ap_mail.mailrcv import MailRcv
from ap_mail.mailsnd import MailDoSnd

TyAny = Any
TyDic = dict[Any, Any]
TyPath = str
TnAny = None | TyAny

TnPath = None | TyPath


class Mail:

    @staticmethod
    def snd(kwargs: TyDic) -> None:
        _in_path_mail_snd: TyPath = kwargs.get('in_path_mail_snd', '')
        LogEq.debug("_in_path_mail_snd", _in_path_mail_snd)
        _path: TnPath = Path.sh_path_by_tpl_pac_sep(_in_path_mail_snd, kwargs)
        LogEq.debug("_path", _path)
        _aod_snd: TnAny = Yaml_.read_with_safeloader(_path)
        if not _aod_snd:
            raise Exception(f"Content of yaml file = {_path} is undefined or empty")
        LogEq.debug("_aod_snd", _aod_snd)

        for _d_snd in _aod_snd:
            MailDoSnd.send(_d_snd, kwargs)

    @staticmethod
    def rcv(kwargs: TyDic) -> None:
        _in_path_mail_rcv = kwargs.get('in_path_mail_rcv', '')
        LogEq.debug("_in_path_mail_rcv", _in_path_mail_rcv)
        _path: TnPath = Path.sh_path_by_tpl_pac_sep(_in_path_mail_rcv, kwargs)
        LogEq.debug("_path", _path)
        _d_rcv: TnAny = Yaml_.read_with_safeloader(_path)
        if not _d_rcv:
            raise Exception(f"Content of yaml file = {_path} is undefined or empty")
        # Connect to the server
        _mail = MailRcv.connect(_d_rcv)
        # Login to your account
        MailRcv.login(_mail, _d_rcv)

        # Select the mailbox you want to check
        _mail.select("inbox")
        # Search for all emails in the inbox
        # status, messages = _mail.search(None, "ALL")
        # Search for unread emails
        status, messages = _mail.search(None, 'UNSEEN')
        # Convert messages to a list of email IDs

        # Reading Emails
        # Now that you have the email IDs, you can fetch and read the emails.
        # Here’s how to do it:
        # Fetch the latest email
        # latest_email_id = _email_ids[-1]
        MailRcv.yield_body(_mail, messages[0])

        # Logout
        _mail.logout()



from ap_mail.mail import Mail
from ut_dic.doc import DoC

from collections.abc import Callable
from typing import Any

TyCallable = Callable[..., Any]
TyDic = dict[Any, Any]
TyDoC = dict[str, TyCallable]

TnDic = None | TyDic


class Task:
    """
    General Task class
    """
    """
    Dictionary of callables of class Setup or Email
    """
    doc: TyDoC = {
        'mailsnd': Mail.snd,
        'mailrcv': Mail.rcv,
    }

    @classmethod
    def do(cls, kwargs: TyDic) -> None:
        """
        Select the task method from the task command table for the given
        command (value of 'cmd' in kwargs) and execute the selected method.
        """
        DoC.ex_cmd(cls.doc, kwargs)
